/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Copyright by The HDF Group.                                               *
 * All rights reserved.                                                      *
 *                                                                           *
 * This file was adapted from the parse.y of the HDF5 code.              *
 * This is used to generate the parse.c by				     *
 *   yacc -pH5EDITyy -o parse.c -d parse.y			     *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

%{
#include<h5edit.h>

extern int yylex();
extern int yyerror(char *);

#define STACK_SIZE      16
#define DATA_SIZE_MAX	512	/* max number of data points */

/* types definitions */
/* nothing yet */

/* variables */
/* Attribute name and handle */
char		*attribute_name = NULL;
hid_t		attribute_id = -1;
char		*attribute_type_name = NULL;
hid_t		attribute_type_id = -1;
H5T_class_t	attribute_type_class = H5T_NO_CLASS;
hid_t		attribute_space_id = -1;
H5S_class_t	attribute_space_class = H5S_NO_CLASS;
H5T_class_t	data_value_class = H5T_NO_CLASS;
struct arr_info	attribute_dims;			/* attribute dimension */


		/* attribute data buffer */
/*void		*attribute_data = NULL; */
		/* this approach does not work for compound types */
union {
    int		data_int[DATA_SIZE_MAX];	/* attribute integer data */
    double	data_float[DATA_SIZE_MAX];	/* attribute float data */
    char	data_string[DATA_SIZE_MAX];	/* attribute string data */
    char	data_char[DATA_SIZE_MAX];	/* for SCALAR string data */
} attribute_data;
int		attribute_data_needed = -1;
int		attribute_data_gotten = -1;
		/* points to available string data buffer */
char*		attribute_data_pt = &(attribute_data.data_string);



/* Dataset name and handle */
char	*group_name = NULL;
char	*dataset_name = NULL;
hid_t	dataset_id = -1;

/*structure for array type information*/
struct arr_info {
    hsize_t	dims[H5S_MAX_RANK];     /*size of each dimension, limited to 32 dimensions*/
    int		ndims;                  /*number of dimensions*/
    hbool_t	is_dim;                 /*flag to lexer for dimension*/
};


/*structure for compound type information*/
struct cmpd_info {
    hid_t       id;             /*type ID*/
    hbool_t     is_field;       /*flag to lexer for compound member*/
    hbool_t     first_memb;     /*flag for first compound member*/
};

/*stack for nested compound type*/
struct cmpd_info h5edit_cmpd_stack[STACK_SIZE] = {
    {0, 0, 1}, {0, 0, 1}, {0, 0, 1}, {0, 0, 1},
    {0, 0, 1}, {0, 0, 1}, {0, 0, 1}, {0, 0, 1},
    {0, 0, 1}, {0, 0, 1}, {0, 0, 1}, {0, 0, 1},
    {0, 0, 1}, {0, 0, 1}, {0, 0, 1}, {0, 0, 1} };

int h5edit_csindex = -1;                /*pointer to the top of compound stack*/

/*stack for nested array type*/
struct arr_info arr_stack[STACK_SIZE];
int h5edit_asindex = -1;               /*pointer to the top of array stack*/ 

hbool_t     h5edit_is_str_size = 0;        /*flag to lexer for string size*/
/*hbool_t     is_str_pad = 0;         */ /*flag to lexer for string padding*/
H5T_pad_t   str_pad;                /*variable for string padding*/
H5T_cset_t  str_cset;               /*variable for string character set*/
hbool_t     h5edit_is_variable = 0;        /*variable for variable-length string*/
int      str_size;               /*variable for string size*/
   

%}
%union {
    int   ival;         /*for integer token*/
    double fval;	/*for floating point token */
    char  *sval;        /*for name string*/
}

/* reserved command keyword tokens */
%token <ival> CREATE_KWORD DELETE_KWORD
%token <ival> ATTRIBUTE_KWORD DATASET_KWORD GROUP_KWORD
%token <ival> DATATYPE_KWORD DATASPACE_KWORD DATA_KWORD
%token <sval> IDENTIFIER
%token <sval> PATHNAME
%token <sval> QUOTED_STRING
%token <sval> DQUOTED_STRING
%token <sval> EOSTMT_TOKEN
%token <sval> WSPACE		/* White spaces */

/* Data types */
%token <sval> INTEGER_TYPE_TOKEN FLOAT_TYPE_TOKEN

%token <ival> H5T_STD_I8BE_TOKEN H5T_STD_I8LE_TOKEN H5T_STD_I16BE_TOKEN  H5T_STD_I16LE_TOKEN
%token <ival> H5T_STD_I32BE_TOKEN H5T_STD_I32LE_TOKEN H5T_STD_I64BE_TOKEN H5T_STD_I64LE_TOKEN
%token <ival> H5T_STD_U8BE_TOKEN H5T_STD_U8LE_TOKEN H5T_STD_U16BE_TOKEN  H5T_STD_U16LE_TOKEN
%token <ival> H5T_STD_U32BE_TOKEN H5T_STD_U32LE_TOKEN H5T_STD_U64BE_TOKEN H5T_STD_U64LE_TOKEN
%token <ival> H5T_NATIVE_CHAR_TOKEN H5T_NATIVE_SCHAR_TOKEN H5T_NATIVE_UCHAR_TOKEN 
%token <ival> H5T_NATIVE_SHORT_TOKEN H5T_NATIVE_USHORT_TOKEN H5T_NATIVE_INT_TOKEN H5T_NATIVE_UINT_TOKEN 
%token <ival> H5T_NATIVE_LONG_TOKEN H5T_NATIVE_ULONG_TOKEN H5T_NATIVE_LLONG_TOKEN H5T_NATIVE_ULLONG_TOKEN

%token <ival> H5T_IEEE_F32BE_TOKEN H5T_IEEE_F32LE_TOKEN H5T_IEEE_F64BE_TOKEN H5T_IEEE_F64LE_TOKEN
%token <ival> H5T_NATIVE_FLOAT_TOKEN H5T_NATIVE_DOUBLE_TOKEN H5T_NATIVE_LDOUBLE_TOKEN

%token <ival> H5T_STRING_TOKEN STRSIZE_TOKEN STRPAD_TOKEN CSET_TOKEN CTYPE_TOKEN H5T_VARIABLE_TOKEN
%token <ival> H5T_STR_NULLTERM_TOKEN H5T_STR_NULLPAD_TOKEN H5T_STR_SPACEPAD_TOKEN 
%token <ival> H5T_CSET_ASCII_TOKEN H5T_CSET_UTF8_TOKEN H5T_C_S1_TOKEN H5T_FORTRAN_S1_TOKEN

%token <ival> H5T_COMPOUND_TOKEN
%token <ival> H5T_ARRAY_TOKEN
%token <ival> H5T_VLEN_TOKEN

%token <sval> STRING_CONST
%token <ival> INTEGER_CONST
%token <fval> FLOAT_CONST
%token <ival> '{' '}' '[' ']' '(' ')' ':' ','

/* data space tokens */
%token <ival> SCALAR_TOKEN SIMPLE_TOKEN NULL_TOKEN

%start		editstmts	/* editstmts */

%%
editstmts	:	/*empty*/
		|	editstmts editstmt
		;

editstmt	: 	editcmd eostmt
		;

editcmd		:	create_cmd 
		|	delete_cmd 
		;

create_cmd	:	/* two kinds of create commands of differnt syntax, create_cmd1 and create_cmd2 */
			create_cmd1
		|	create_cmd2
		;

delete_cmd	:	/* two kinds of delete commands of differnt syntax, delete_cmd1 and delete_cmd2 */
			delete_cmd1
		|	delete_cmd2
		;

create_cmd1	:	/* create command with a single combined targetobject/attribute */
			CREATE_KWORD targetobject attribute_definition
			{
			    if (get_debug()) printf("found create commnad syntax 1\n");
			    /* error check */
			    if (!dataset_name && !group_name){
				fprintf(stderr, "neither dataset nor group name is provided for CREATE command\n");
				leave(EXIT_FAILURE);
			    }
			    /* Execute the CREATE command */
			    /* Sending NULL for attribute_name because it is not defined. */
			    if (0 != create_attribute(NULL, group_name, dataset_name,
				attribute_type_id, attribute_space_id, &attribute_data)){
				CMD_ERROR("CREATE Attribute command failed");
			    }
			    /* Cleanup */
			    if (group_name){
				HDfree(group_name);
				group_name = NULL;
			    }
			    if (dataset_name){
				HDfree(dataset_name);
				dataset_name = NULL;
			    }
			    attribute_data_needed = -1;

			    if (attribute_space_id >= 0){
				H5Sclose(attribute_space_id);
				attribute_space_id = -1;
			    }
			    if (attribute_type_id >= 0){
				H5Tclose(attribute_type_id);
				attribute_type_id = -1;
			    }
			    attribute_type_class = H5T_NO_CLASS;
			}
		;

create_cmd2	:	CREATE_KWORD targetobject attributename attribute_definition
			{
			    if (get_debug()) printf("found create commnad\n");
			    /* error check */
			    if (!attribute_name){
				fprintf(stderr, "attribute name missing from CREATE command\n");
				leave(EXIT_FAILURE);
			    }
			    if (!dataset_name && !group_name){
				fprintf(stderr, "neither dataset nor group name is provided for CREATE command\n");
				leave(EXIT_FAILURE);
			    }
			    /* Execute the CREATE command */
			    if (0 != create_attribute(attribute_name, group_name, dataset_name,
				attribute_type_id, attribute_space_id, &attribute_data)){
				CMD_ERROR("CREATE Attribute command failed");
			    }
			    /* Cleanup */
			    HDfree(attribute_name);
			    attribute_name = NULL;
			    if (group_name){
				HDfree(group_name);
				group_name = NULL;
			    }
			    if (dataset_name){
				HDfree(dataset_name);
				dataset_name = NULL;
			    }
			    attribute_data_needed = -1;

			    if (attribute_space_id >= 0){
				H5Sclose(attribute_space_id);
				attribute_space_id = -1;
			    }
			    if (attribute_type_id >= 0){
				H5Tclose(attribute_type_id);
				attribute_type_id = -1;
			    }
			    attribute_type_class = H5T_NO_CLASS;
			}
		;

delete_cmd1	:	/* delete command with a single combined targetobject/attribute */
			DELETE_KWORD targetobject
			{
			    if (get_debug()) printf("found delete commnad syntax 1\n");
			    /* error check */
			    if (!dataset_name && !group_name){
				fprintf(stderr, "neither dataset nor group name is provided for DELETE command\n");
				leave(EXIT_FAILURE);
			    }
			    /* Execute the DELETE command */
			    /* Sending NULL for attribute_name because it is not defined. */
			    if (0 != delete_attribute(NULL, group_name, dataset_name)){
				CMD_ERROR("DELETE Attribute command failed");
			    }
			    /* Cleanup */
			    if (group_name){
				HDfree(group_name);
				group_name = NULL;
			    }
			    if (dataset_name){
				HDfree(dataset_name);
				dataset_name = NULL;
			    }
			}

delete_cmd2	:	/* delete command with separated targetobject and attribute */
			DELETE_KWORD targetobject attributename
			{
			    if (get_debug()) printf("found delete commnad\n");
			    /* error check */
			    if (!attribute_name){
				fprintf(stderr, "attribute name missing from DELETE command\n");
				leave(EXIT_FAILURE);
			    }
			    if (!dataset_name && !group_name){
				fprintf(stderr, "neither dataset nor group name is provided for DELETE command\n");
				leave(EXIT_FAILURE);
			    }
			    /* Execute the DELETE command */
			    if (0 != delete_attribute(attribute_name, group_name, dataset_name)){
				CMD_ERROR("DELETE Attribute command failed");
			    }
			    /* Cleanup */
			    HDfree(attribute_name);
			    attribute_name = NULL;
			    if (group_name){
				HDfree(group_name);
				group_name = NULL;
			    }
			    if (dataset_name){
				HDfree(dataset_name);
				dataset_name = NULL;
			    }
			}

attributename	:	objectname
			{if (get_debug()) printf("got attribute name(%s)\n", H5EDITyylval.sval);
			    /* save the attribute name */
			    if (attribute_name != NULL){
				fprintf(stderr, "Already got an attribute name\n");
				leave(EXIT_FAILURE);
			    }
			    if ((attribute_name = HDstrdup(H5EDITyylval.sval)) == NULL){
				fprintf(stderr, "strdup failed\n");
				leave(EXIT_FAILURE);
			    }
			}
		;

targetobject	:	datasetname
		|	DATASET_KWORD datasetname
		|	GROUP_KWORD	groupname
		;

datasetname	:	objectname
			{if (get_debug()) printf("got dataset name(%s)\n", H5EDITyylval.sval);
			    /* save the dataset name */
			    if (dataset_name != NULL){
				fprintf(stderr, "Already got an dataset name\n");
				leave(EXIT_FAILURE);
			    }
			    if ((dataset_name = HDstrdup(H5EDITyylval.sval)) == NULL){
				fprintf(stderr, "strdup failed\n");
				leave(EXIT_FAILURE);
			    }
			}
		;

groupname	:	objectname
			{if (get_debug()) printf("got group name(%s)\n", H5EDITyylval.sval);
			    /* save the group name */
			    if (group_name != NULL){
				fprintf(stderr, "Already got an group name\n");
				leave(EXIT_FAILURE);
			    }
			    if ((group_name = HDstrdup(H5EDITyylval.sval)) == NULL){
				fprintf(stderr, "strdup failed\n");
				leave(EXIT_FAILURE);
			    }
			}
		;

objectname	:	IDENTIFIER
		|	PATHNAME
		|	QUOTED_STRING
		|	DQUOTED_STRING
		;

eostmt		:	EOSTMT_TOKEN
		;

/* Attribute Definition */
attribute_definition : "{"
			attribute_body_definition
			"}"
			{ if (get_debug()) printf("found attribute definition\n");

			}
		;

attribute_body_definition:
			datatype_stmt dataspace_stmt data_stmt
		|	datatype_stmt                data_stmt
		|		      dataspace_stmt data_stmt
		|	                             data_stmt
		;

datatype_stmt	:	DATATYPE_KWORD datatype_definition
		|	datatype_definition
		;

datatype_definition:	datatype_class
		;

datatype_class	:	integer_class
		|	floatpoint_class
		|	string_class
		;

string_class	:	H5T_STRING_TOKEN "{" string_type_definition "}"
			{
			    if (get_debug()) printf("found string type\n");
			    if (attribute_type_class != H5T_NO_CLASS){
				fprintf(stderr, "Already has an attribute data type\n");
				leave(EXIT_FAILURE);
			    }
			    attribute_type_class = H5T_STRING;
			    /* for now, assume it is null-terminated strings */
			    if ((attribute_type_id=H5Tcopy(H5T_C_S1)) < 0){
				fprintf(stderr, "Attribute string type create failure\n");
				leave(EXIT_FAILURE);
			    }
			    if (H5Tset_size(attribute_type_id,str_size) < 0){
				fprintf(stderr, "Attribute string size set failure\n");
				leave(EXIT_FAILURE);
			    }
			    if (H5Tset_strpad(attribute_type_id, H5T_STR_NULLTERM) < 0){
				fprintf(stderr, "Attribute string pad set failure\n");
				leave(EXIT_FAILURE);
			    }
			}
		;

string_type_definition : string_size_definition
			{
			    if (get_debug()) printf("found string size\n");
			}
		;

string_size_definition : STRSIZE_TOKEN INTEGER_CONST
			{   /* Must test value range before assign it to str_zize */
			    /* which is size_t type which is unsigned. */
			    if (get_debug()) printf("String size is (%d)\n",
				    H5EDITyylval.ival);
			    if (H5EDITyylval.ival <= 0){
				fprintf(stderr, "String size must be positive (%d)\n",
				    H5EDITyylval.ival);
				leave(EXIT_FAILURE);
			    }
			    str_size = H5EDITyylval.ival;
			}
		;

string_const	:	QUOTED_STRING
		|	DQUOTED_STRING
		;

integer_class	:	INTEGER_TYPE_TOKEN
			{
			    if (get_debug()) printf("found integer type(%s)\n",
				H5EDITyylval.sval);
			    if (attribute_type_class != H5T_NO_CLASS){
				fprintf(stderr, "Already has an attribute data type\n");
				leave(EXIT_FAILURE);
			    }
			    attribute_type_class = H5T_INTEGER;
			    if ((attribute_type_id=H5LTtext_to_dtype(H5EDITyylval.sval, H5LT_DDL)) < 0){
				fprintf(stderr, "Attribute type create failure\n");
				leave(EXIT_FAILURE);
			    }
			}
		;

floatpoint_class :	FLOAT_TYPE_TOKEN
			{
			    if (get_debug()) printf("found float type(%s)\n",
				H5EDITyylval.sval);
			    if (attribute_type_class != H5T_NO_CLASS){
				fprintf(stderr, "Already has an attribute data type\n");
				leave(EXIT_FAILURE);
			    }
			    attribute_type_class = H5T_FLOAT;
			    if ((attribute_type_id=H5LTtext_to_dtype(H5EDITyylval.sval, H5LT_DDL)) < 0){
				fprintf(stderr, "Attribute type create failure\n");
				leave(EXIT_FAILURE);
			    }
			}
		;

dataspace_stmt	:	DATASPACE_KWORD dataspace_definition
		|	dataspace_definition
		;

dataspace_definition:
			dataspace_simple
		|	dataspace_scalar
		|	dataspace_null
		;

dataspace_scalar:	SCALAR_TOKEN
			{
			    if (get_debug()) printf("found scalar dataspace\n");
			    attribute_space_class = H5S_SCALAR;
			    if (attribute_space_id != -1){
				fprintf(stderr, "Already has an attribute space\n");
				leave(EXIT_FAILURE);
			    }
			    if ((attribute_space_id=H5Screate(H5S_SCALAR)) < 0){
				fprintf(stderr, "Attribute space create failure\n");
				leave(EXIT_FAILURE);
			    }
			    attribute_data_needed = 1;
			}
		;

dataspace_simple:	SIMPLE_TOKEN dataspace_dims_list
		|	dataspace_dims_list
		;

dataspace_dims_list:

			{
			    if (get_debug()) printf("looking for dimlist\n");
			    attribute_space_class = H5S_SIMPLE;
			    /* reset attribute dimension information */
			    HDmemset(&attribute_dims, 0, sizeof(attribute_dims));
			}

			"(" dim_firstone dims_list ")"
			{
			    if (get_debug()){
				int i;
				printf("found simple dataspace\n");
				printf("rank=%d:\n", attribute_dims.ndims);
				for (i=0; i<attribute_dims.ndims; i++){
				    printf("%ld\n", (long) attribute_dims.dims[i]);
				}
			    }
			    if (attribute_space_id != -1){
				fprintf(stderr, "Already has an attribute space\n");
				leave(EXIT_FAILURE);
			    }
			    if ((attribute_space_id=H5Screate_simple(attribute_dims.ndims,
				    &(attribute_dims.dims), NULL)) < 0 ){
				fprintf(stderr, "Attribute space create failure\n");
				leave(EXIT_FAILURE);
			    }
			    /* Calculate number of data elements needed */
			    attribute_data_needed = attribute_dims.dims[0];
			    {   int i;
				for (i=1; i < attribute_dims.ndims; i++){
				    attribute_data_needed *= attribute_dims.dims[i];
				}
				if (get_debug()) printf("attribute data needed=%d\n",
					attribute_data_needed);
			    }
			    /* AKC: I don't like the coding here. */
			    /* It is repeating code.  Need to fix it later. */
			    /* Default datatype is Native Float */
			    if ( attribute_type_class == H5T_NO_CLASS){
				attribute_type_class = H5T_FLOAT;
				if ((attribute_type_id=H5Tcopy(H5T_NATIVE_FLOAT)) < 0){
				    fprintf(stderr, "Attribute default type create failure\n");
				    leave(EXIT_FAILURE);
				}
			    }
			    switch (attribute_type_class){
			    case H5T_INTEGER:
			    case H5T_FLOAT:
				if (attribute_data_needed > DATA_SIZE_MAX){
				    fprintf(stderr,
					"Too many attribute data needed(%d). "
					"Maximum is %d\n",
					attribute_data_needed, DATA_SIZE_MAX);
				    leave(EXIT_FAILURE);
				}
				break;
			    case H5T_STRING:
				if ((attribute_data_needed*str_size) > sizeof(attribute_data)){
				    fprintf(stderr,
					"Too much attribute string data needed(%d). "
					"Maximum is %d\n",
					(int)attribute_data_needed*str_size,
					sizeof(attribute_data));
				    leave(EXIT_FAILURE);
				}
				break;
			    default:
				fprintf(stderr, "Unsupported data type(%d)\n",
				    attribute_type_class);
				leave(EXIT_FAILURE);
			    }
			}
		;

dim_firstone	:	INTEGER_CONST
			{   /* first dimension */
			    if (H5EDITyylval.ival <=0){
				fprintf(stderr, "Dimension must be positive. Got %d.\n",
				    H5EDITyylval.ival);
				leave(EXIT_FAILURE);
			    }
			    attribute_dims.dims[attribute_dims.ndims] = H5EDITyylval.ival;
			    attribute_dims.ndims++;
			}
		;

dims_list	:	/* empty */
		|	dims_list dim_more
		;

dim_more	:	"," INTEGER_CONST
			{   /* more dimensions */
			    if (attribute_dims.ndims >= H5S_MAX_RANK){
				fprintf(stderr, "Too many dimensions. Maximun is %d\n",
				    H5S_MAX_RANK);
				leave(EXIT_FAILURE);
			    }
			    if (H5EDITyylval.ival <=0){
				fprintf(stderr, "Dimension must be positive. Got %d.\n",
				    H5EDITyylval.ival);
				leave(EXIT_FAILURE);
			    }
			    attribute_dims.dims[attribute_dims.ndims] = H5EDITyylval.ival;
			    attribute_dims.ndims++;
			}
		;

dataspace_null	:	NULL_TOKEN
		;

			
data_stmt	:	DATA_KWORD data_definition
		|	data_definition
		;

data_definition :	"{"
			{
			    /* get the data buffer ready for new data */
			    if (get_debug()) printf("found attribute data\n");
			    attribute_data_gotten = 0;
			    attribute_data_pt = &(attribute_data.data_string);
			    /* Default datatype is Native Float */
			    if ( attribute_type_class == H5T_NO_CLASS){
				attribute_type_class = H5T_FLOAT;
				if ((attribute_type_id=H5Tcopy(H5T_NATIVE_FLOAT)) < 0){
				    fprintf(stderr, "Attribute default type create failure\n");
				    leave(EXIT_FAILURE);
				}
			    }
			    /* Default dataspace is SCALAR */
			    if (attribute_space_id == -1){
				attribute_space_class = H5S_SCALAR;
				if ((attribute_space_id=H5Screate(H5S_SCALAR)) < 0){
				    fprintf(stderr, "Attribute default space create failure\n");
				    leave(EXIT_FAILURE);
				}
				attribute_data_needed = 1;
			    }
			}
			data_firstval data_list
			{
			    if (attribute_data_gotten < attribute_data_needed){
				fprintf(stderr, "Fewer data than needed. "
				    "Need %d but got %d data only\n",
				    attribute_data_needed, attribute_data_gotten);
				leave(EXIT_FAILURE);
			    }
			}
			"}"
			{ /* no action */
			}
		;

data_firstval	:	data_val
		;

data_list	:	/* empty */
		|	data_list data_more
		;

data_more	:	"," data_val
		;

data_val	:	INTEGER_CONST
			{
			    if (attribute_data_gotten >= attribute_data_needed){
				fprintf(stderr, "More data than needed. Only %d are needed.\n",
				    attribute_data_needed);
				leave(EXIT_FAILURE);
			    }
			    if (attribute_type_class != H5T_INTEGER){
				fprintf(stderr, "Gotten incorrect type of data\n");
				leave(EXIT_FAILURE);
			    }
			    attribute_data.data_int[attribute_data_gotten++] = H5EDITyylval.ival;
			}

		|	FLOAT_CONST
			{
			    if (attribute_data_gotten >= attribute_data_needed){
				fprintf(stderr, "More data than needed. Only %d are needed.\n",
				    attribute_data_needed);
				leave(EXIT_FAILURE);
			    }
			    if (attribute_type_class != H5T_FLOAT){
				fprintf(stderr, "Gotten incorrect type of data\n");
				leave(EXIT_FAILURE);
			    }
			    attribute_data.data_float[attribute_data_gotten++] = H5EDITyylval.fval;
			}
		|	string_const
			{
			    if (attribute_data_gotten >= attribute_data_needed){
				fprintf(stderr, "More data than needed. Only %d are needed.\n",
				    attribute_data_needed);
				leave(EXIT_FAILURE);
			    }
			    if (attribute_type_class != H5T_STRING){
				fprintf(stderr, "Gotten incorrect type of data\n");
				leave(EXIT_FAILURE);
			    }
			    switch (attribute_space_class) {
			    case H5S_SCALAR:
				/* this works for scalar only. Copy the
				 * string_const to attribute_data. */
				if (attribute_data_gotten > 0 ){
				    fprintf(stderr, "More data than needed. Only %d are needed.\n",
					attribute_data_needed);
				    leave(EXIT_FAILURE);
				}
				HDstrncpy(attribute_data_pt, H5EDITyylval.sval, str_size);
				/* move data pointer forward */
				attribute_data_pt += str_size;
				attribute_data_gotten++;
				break;
			    case H5S_SIMPLE:
				/* This should work for SIMPLE space */
				/* this is not working
				fprintf(stderr, "not able to support string type of SIMPLE space\n");
				leave(EXIT_FAILURE);
				*/
				HDstrncpy(attribute_data_pt, H5EDITyylval.sval, str_size);
				/* move data pointer forward */
				attribute_data_pt += str_size;
				attribute_data_gotten++;
				break;
			    default:
				fprintf(stderr, "Do not know how to store string_const "
				    "of this kind of data space(%d)\n",
				    attribute_space_class);
				leave(EXIT_FAILURE);
			    }
			}
		;


